@@ -1,16 +1,16 @@
-from functools import wraps, partial
-from typing import Union, AnyStr, ByteString, List, Sequence, Any
 import warnings
+from functools import wraps
+from typing import AnyStr, ByteString, List, Sequence, Union
 
-from redis import StrictRedis
 import numpy as np
+from deprecated import deprecated
+from redis import StrictRedis
 
 from redisai import command_builder as builder
 from redisai.dag import Dag
 from redisai.pipeline import Pipeline
 from redisai.postprocessor import Processor
 
-
 processor = Processor()
 
 
@@ -65,12 +65,17 @@ class Client(StrictRedis):
             self.enable_postprocess,
             self.connection_pool,
             self.response_callbacks,
-            transaction=True,
-            shard_hint=None,
+            transaction=transaction,
+            shard_hint=shard_hint,
         )
 
     def dag(
-        self, load: Sequence = None, persist: Sequence = None, readonly: bool = False
+        self,
+        load: Sequence = None,
+        persist: Sequence = None,
+        routing: AnyStr = None,
+        timeout: int = None,
+        readonly: bool = False
     ) -> "Dag":
         """
         It returns a DAG object on which other DAG-allowed operations can be called. For
@@ -81,7 +86,16 @@ class Client(StrictRedis):
         load : Union[AnyStr, List[AnyStr]]
             Load the list of given values from the keyspace to DAG scope
         persist : Union[AnyStr, List[AnyStr]]
-            Write the list of given key, values to the keyspace from DAG scope
+            For each tensor key in the given list, write its values to the keyspace from
+            DAG scope after the DAG execution is finished.
+        routing : AnyStr
+            Denotes a key to be used in the DAG or a tag that will assist in routing the dag
+            execution command to the right shard. Redis will verify that all potential key
+            accesses are done to within the target shard.
+        timeout : int
+            The max number on milisecinds that may pass before the request is prossced
+            (meaning that the result will not be computed after that time and TIMEDOUT
+            is returned in that case)
         readonly : bool
             If True, it triggers AI.DAGRUN_RO, the read only DAG which cannot write (PERSIST) to
             the keyspace. But since it can't write, it can execute on replicas
@@ -96,7 +110,7 @@ class Client(StrictRedis):
         -------
         >>> con.tensorset('tensor', ...)
         'OK'
-        >>> con.modelset('model', ...)
+        >>> con.modelstore('model', ...)
         'OK'
         >>> dag = con.dag(load=['tensor'], persist=['output'])
         >>> dag.tensorset('another', ...)
@@ -105,9 +119,7 @@ class Client(StrictRedis):
         >>> # You can even chain the operations
         >>> result = dag.tensorset(**akwargs).modelrun(**bkwargs).tensorget(**ckwargs).run()
         """
-        return Dag(
-            load, persist, self.execute_command, readonly, self.enable_postprocess
-        )
+        return Dag(load, persist, routing, timeout, self.execute_command, readonly)
 
     def loadbackend(self, identifier: AnyStr, path: AnyStr) -> str:
         """
@@ -136,6 +148,83 @@ class Client(StrictRedis):
         res = self.execute_command(*args)
         return res if not self.enable_postprocess else processor.loadbackend(res)
 
+    def modelstore(
+        self,
+        key: AnyStr,
+        backend: str,
+        device: str,
+        data: ByteString,
+        batch: int = None,
+        minbatch: int = None,
+        minbatchtimeout: int = None,
+        tag: AnyStr = None,
+        inputs: Union[AnyStr, List[AnyStr]] = None,
+        outputs: Union[AnyStr, List[AnyStr]] = None,
+    ) -> str:
+        """
+        Set the model on provided key.
+
+        Parameters
+        ----------
+        key : AnyStr
+            Key name
+        backend : str
+            Backend name. Allowed backends are TF, TORCH, TFLITE, ONNX
+        device : str
+            Device name. Allowed devices are CPU and GPU. If multiple GPUs are available,
+            it can be specified using the format GPU:<gpu number>. For example: GPU:0
+        data : bytes
+            Model graph read as bytes string
+        batch : int
+            Number of batches for doing auto-batching
+        minbatch : int
+            Minimum number of samples required in a batch for model execution
+        minbatchtimeout : int
+            The max number of miliseconds for which the engine will not trigger an execution
+            if the number of samples is lower than minbatch (after minbatchtimeout is passed,
+            the execution will start even if minbatch jas not reached)
+        tag : AnyStr
+            Any string that will be saved in RedisAI as tag for the model
+        inputs : Union[AnyStr, List[AnyStr]]
+            Input node(s) in the graph. Required only Tensorflow graphs
+        outputs : Union[AnyStr, List[AnyStr]]
+            Output node(s) in the graph Required only for Tensorflow graphs
+
+        Returns
+        -------
+        str
+            'OK' if success, raise an exception otherwise
+
+        Example
+        -------
+        >>> # Torch model
+        >>> model_path = os.path.join('path/to/TorchScriptModel.pt')
+        >>> model = open(model_path, 'rb').read()
+        >>> con.modeltore("model", 'torch', 'cpu', model, tag='v1.0')
+        'OK'
+        >>> # Tensorflow model
+        >>> model_path = os.path.join('/path/to/tf_frozen_graph.pb')
+        >>> model = open(model_path, 'rb').read()
+        >>> con.modelstore('m', 'tf', 'cpu', model,
+        ...              inputs=['a', 'b'], outputs=['mul'], tag='v1.0')
+        'OK'
+        """
+        args = builder.modelstore(
+            key,
+            backend,
+            device,
+            data,
+            batch,
+            minbatch,
+            minbatchtimeout,
+            tag,
+            inputs,
+            outputs,
+        )
+        res = self.execute_command(*args)
+        return res if not self.enable_postprocess else processor.modelstore(res)
+
+    @deprecated(version="1.2.0", reason="Use modelstore instead")
     def modelset(
         self,
         key: AnyStr,
@@ -247,6 +336,56 @@ class Client(StrictRedis):
         res = self.execute_command(*args)
         return res if not self.enable_postprocess else processor.modeldel(res)
 
+    def modelexecute(
+        self,
+        key: AnyStr,
+        inputs: Union[AnyStr, List[AnyStr]],
+        outputs: Union[AnyStr, List[AnyStr]],
+        timeout: int = None,
+    ) -> str:
+        """
+        Run the model using input(s) which are already in the scope and are associated
+        to some keys. Modelexecute also needs the output key name(s) to store the output
+        from the model. The number of outputs from the model and the number of keys
+        provided here must be same. Otherwise, RedisAI throws an error
+
+        Parameters
+        ----------
+        key : str
+            Model key to run
+        inputs : Union[AnyStr, List[AnyStr]]
+            Tensor(s) which is already saved in the RedisAI using a tensorset call. These
+            tensors will be used as the inputs for the modelexecute
+        outputs : Union[AnyStr, List[AnyStr]]
+            keys on which the outputs to be saved. If those keys exist already,
+            modelexecute will overwrite them with new values
+        timeout : int
+            The max number on milisecinds that may pass before the request is prossced
+            (meaning that the result will not be computed after that time and TIMEDOUT
+            is returned in that case
+
+        Returns
+        -------
+        str
+            'OK' if success, raise an exception otherwise
+
+        Example
+        -------
+        >>> con.modelstore('m', 'tf', 'cpu', model_pb,
+        ...              inputs=['a', 'b'], outputs=['mul'], tag='v1.0')
+        'OK'
+        >>> con.tensorset('a', (2, 3), dtype='float')
+        'OK'
+        >>> con.tensorset('b', (2, 3), dtype='float')
+        'OK'
+        >>> con.modelexecute('m', ['a', 'b'], ['c'])
+        'OK'
+        """
+        args = builder.modelexecute(key, inputs, outputs, timeout)
+        res = self.execute_command(*args)
+        return res if not self.enable_postprocess else processor.modelexecute(res)
+
+    @deprecated(version="1.2.0", reason="Use modelexecute instead")
     def modelrun(
         self,
         key: AnyStr,
@@ -277,7 +416,7 @@ class Client(StrictRedis):
 
         Example
         -------
-        >>> con.modelset('m', 'tf', 'cpu', model_pb,
+        >>> con.modelstore('m', 'tf', 'cpu', model_pb,
         ...              inputs=['a', 'b'], outputs=['mul'], tag='v1.0')
         'OK'
         >>> con.tensorset('a', (2, 3), dtype='float')
@@ -401,6 +540,66 @@ class Client(StrictRedis):
             else processor.tensorget(res, as_numpy, as_numpy_mutable, meta_only)
         )
 
+    def scriptstore(
+        self, key: AnyStr, device: str, script: str, entry_points: Union[str, Sequence[str]], tag: AnyStr = None
+    ) -> str:
+        """
+        Set the script to RedisAI. The difference from scriptset is that in scriptstore
+        you must specify entry points within your script. These functions must have specific
+        signature: 'def entry_point(tensors: List[Tensor], keys: List[str], args: List[str])'.
+        RedisAI uses the TorchScript engine to execute the script. So the script should
+        have only TorchScript supported constructs. That being said, it's important to
+        mention that using redisai script to do post processing or pre processing for a
+        Tensorflow (or any other backend) is completely valid. For more details about
+        TorchScript and supported ops, checkout TorchScript documentation.
+
+        Parameters
+        ----------
+        key : AnyStr
+            Script key at the server
+        device : str
+            Device name. Allowed devices are CPU and GPU. If multiple GPUs are available.
+            it can be specified using the format GPU:<gpu number>. For example: GPU:0
+        script : str
+            Script itself, as a Python string
+        entry_points : Union[str, Sequence[str]]
+            A list of functions in the script that may serve as entry point for the
+            execution. Each entry point must have the specify signature:
+            def entry_point(tensors: List[Tensor], keys: List[str], args: List[str]))
+            Note that the script may contain additional helper functions that doesn't
+            have to follow this signature.
+        tag : AnyStr
+            Any string that will be saved in RedisAI as tag for the script
+
+        Returns
+        -------
+        str
+            'OK' if success, raise an exception otherwise
+
+        Note
+        ----
+        Even though ``script`` is pure Python code, it's a subset of Python language and not
+        all the Python operations are supported. For more details, checkout TorchScript
+        documentation. It's also important to note that that the script is executed on a high
+        performance C++ runtime instead of the Python interpreter. And hence ``script`` should
+        not have any import statements (A common mistake people make all the time)
+
+        Example
+        -------
+        >>> script = r'''
+        >>> def bar(tensors: List[Tensor], keys: List[str], args: List[str]):
+        >>>     a = tensors[0]
+        >>>     b = tensors[1]
+        >>>     return a + b
+        >>>'''
+        >>> con.scriptstore('ket', 'cpu', script, 'bar')
+        'OK'
+        """
+        args = builder.scriptstore(key, device, script, entry_points, tag)
+        res = self.execute_command(*args)
+        return res if not self.enable_postprocess else processor.scriptstore(res)
+
+    @deprecated(version="1.2.0", reason="Use scriptstore instead")
     def scriptset(
         self, key: AnyStr, device: str, script: str, tag: AnyStr = None
     ) -> str:
@@ -495,10 +694,11 @@ class Client(StrictRedis):
         res = self.execute_command(*args)
         return res if not self.enable_postprocess else processor.scriptdel(res)
 
+    @deprecated(version="1.2.0", reason="Use scriptexecute instead")
     def scriptrun(
         self,
         key: AnyStr,
-        function: AnyStr,
+        function: str,
         inputs: Union[AnyStr, Sequence[AnyStr]],
         outputs: Union[AnyStr, Sequence[AnyStr]],
     ) -> str:
@@ -509,13 +709,13 @@ class Client(StrictRedis):
         ----------
         key : AnyStr
             Script key
-        function : AnyStr
+        function : str
             Name of the function in the ``script``
         inputs : Union[AnyStr, List[AnyStr]]
             Tensor(s) which is already saved in the RedisAI using a tensorset call. These
             tensors will be used as the input for the modelrun
         outputs : Union[AnyStr, List[AnyStr]]
-            keys on which the outputs to be saved. If those keys exist already, modelrun
+            keys on which the outputs to be saved. If those keys exist already, scriptrun
             will overwrite them with new values
 
         Returns
@@ -532,6 +732,62 @@ class Client(StrictRedis):
         res = self.execute_command(*args)
         return res if not self.enable_postprocess else processor.scriptrun(res)
 
+    def scriptexecute(
+        self,
+        key: AnyStr,
+        function: str,
+        keys: Union[AnyStr, Sequence[AnyStr]] = None,
+        inputs: Union[AnyStr, Sequence[AnyStr]] = None,
+        args: Union[AnyStr, Sequence[AnyStr]] = None,
+        outputs: Union[AnyStr, Sequence[AnyStr]] = None,
+        timeout: int = None,
+    ) -> str:
+        """
+        Run an already set script. Similar to modelexecute.
+        Must specify keys or inputs.
+
+        Parameters
+        ----------
+        key : AnyStr
+            Script key
+        function : str
+            Name of the function in the ``script``
+        keys : Union[AnyStr, Sequence[AnyStr]]
+            Denotes the list of Redis key names that the script will access to
+            during its execution, for both read and/or write operations.
+        inputs : Union[AnyStr, Sequence[AnyStr]]
+            Denotes the input tensors list.
+        args : Union[AnyStr, Sequence[AnyStr]]
+            Denotes the list of additional arguments that a user can send to the
+            script. All args are sent as strings, but can be casted to other types
+            supported by torch script, such as int, or float.
+        outputs : Union[AnyStr, List[AnyStr]]
+            Denotes the output tensors keys' list. If those keys exist already,
+            scriptexecute will overwrite them with new values.
+        timeout : int
+            The max number on milisecinds that may pass before the request is prossced
+            (meaning that the result will not be computed after that time and TIMEDOUT
+            is returned in that case).
+
+        Returns
+        -------
+        str
+            'OK' if success, raise an exception otherwise
+
+        Example
+        -------
+        >>> con.scriptexecute('myscript', 'bar', inputs=['a', 'b'], outputs=['c'])
+        'OK'
+        >>> con.scriptexecute('myscript{tag}', 'addn',
+        >>>                   inputs=['mytensor1{tag}', 'mytensor2{tag}', 'mytensor3{tag}'],
+        >>>                   args=['5.0'],
+        >>>                   outputs=['result{tag}'])
+        'OK'
+        """
+        args = builder.scriptexecute(key, function, keys, inputs, args, outputs, timeout)
+        res = self.execute_command(*args)
+        return res if not self.enable_postprocess else processor.scriptexecute(res)
+
     def scriptscan(self) -> List[List[AnyStr]]:
         """
         Returns the list of all the script in the RedisAI server. Scriptscan API is
